# -*- coding: utf-8 -*-
import pytest

from cglims.apptag import ApplicationTag


@pytest.fixture
def apptag_wgs():
    raw_tag = 'WGSPCFC030'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_wes():
    raw_tag = 'EXOSXTR100'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_microbial():
    raw_tag = 'MWGNXTR003'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_rna():
    raw_tag = 'RNLPOAR030'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_external():
    raw_tag = 'EXXCUSR000'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_external_wgs():
    raw_tag = 'WGXCUSR000'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_metagenome():
    raw_tag = 'METPCFR020'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_rml():
    raw_tag = 'RMLP10R150'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptag_focused_exome():
    raw_tag = 'EFTSXTR020'
    return ApplicationTag(raw_tag)


@pytest.fixture
def apptags():
    return {
        'wgs': apptag_wgs(),
        'wes': apptag_wes(),
        'microbial': apptag_microbial(),
        'rna': apptag_rna(),
        'external': apptag_external(),
        'external_wgs': apptag_external_wgs(),
        'metagenome': apptag_metagenome(),
        'rml': apptag_rml(),
        'targeted': apptag_focused_exome()
    }




# -*- coding: utf-8 -*-
from __future__ import division

# for WGS, how many reads needed to cover genome 1x
READS_PER_1X = 650000000 / 0.75 / 30

SEX_MAP = {'F': 'female', 'M': 'male', 'Unknown': 'unknown', 'unknown': 'unknown'}



# -*- coding: utf-8 -*-
"""Handle gene panels."""

COLLABORATORS = ('cust000', 'cust002', 'cust003', 'cust004', 'cust042')
MASTER_LIST = ('ENDO', 'EP', 'IEM', 'IBMFS', 'mtDNA', 'MIT', 'PEDHEP', 'OMIM-AUTO',
               'PIDCAD', 'PID', 'SKD', 'NMD', 'ATX', 'CTD', 'ID', 'SPG', 'Ataxi', 'AD-HSP')
COMBOS = {
    'DSD': ('DSD', 'HYP', 'SEXDIF', 'SEXDET'),
    'CM': ('CNM', 'CM'),
    'Horsel': ('Horsel', '141217', '141201'),
}


def convert_panels(customer, default_panels):
    """Convert between default panels and all panels included in gene list."""
    if customer in COLLABORATORS:
        # check if all default panels are part of master list
        if all(panel in MASTER_LIST for panel in default_panels):
            return MASTER_LIST

    # the rest are handled the same way
    all_panels = set(default_panels)

    # fill in extra panels if selection is part of a combo
    for panel in default_panels:
        if panel in COMBOS:
            for extra_panel in COMBOS[panel]:
                all_panels.add(extra_panel)

    # add OMIM to every panel choice
    all_panels.add('OMIM-AUTO')

    return list(all_panels)



# -*- coding: utf-8 -*-
from datetime import datetime
import copy
import logging

import click
from dateutil.parser import parse as parse_date
import yaml

from cglims import api
from cglims.constants import SEX_MAP

log = logging.getLogger(__name__)


@click.command()
@click.argument('customer_or_case')
@click.argument('family_id', required=False)
@click.pass_context
def export(context, customer_or_case, family_id):
    """Parse out interesting data about a case."""
    lims = api.connect(context.obj)
    if family_id:
        customer = customer_or_case
    else:
        customer, family_id = customer_or_case.split('-', 1)
    lims_samples = lims.case(customer, family_id)
    case_data = export_case(lims, lims_samples)

    raw_dump = yaml.safe_dump(case_data, default_flow_style=False,
                              allow_unicode=True)
    click.echo(raw_dump)


def export_case(lims_api, lims_samples):
    """Gather data about a case, multiple samples in LIMS."""
    families = (get_familydata(lims_sample) for lims_sample in lims_samples)
    samples = []
    for lims_sample in lims_samples:
        artifacts = lims_api.get_artifacts(samplelimsid=lims_sample.id)
        data = sample_data(lims_api, lims_sample, artifacts)
        samples.append(data)

    family_data = consolidate_family(families)
    family_data['samples'] = list(samples)
    return family_data


def get_familydata(lims_sample):
    """Parse out common (family-level) data."""
    data = {
        'customer': lims_sample.udf['customer'],
        'family_id': lims_sample.udf['familyID'],
        'case_id': "{}-{}".format(lims_sample.udf['customer'],
                                  lims_sample.udf['familyID']),
        'gene_panels': lims_sample.udf['Gene List'].split(';'),
        'reference_genome': lims_sample.udf.get('Reference Genome', 'hg19'),
    }
    return data


def sample_data(lims_api, lims_sample, artifacts):
    """Parse out sample specific data."""
    capture_kit = lims_sample.udf.get('Capture Library version')
    received_at = lims_api.get_received_date(lims_sample.id)
    try:
        data = {
            'id': lims_sample.id,
            'name': lims_sample.name,
            'received_at': received_at,
            'status': lims_sample.udf['Status'].lower(),
            'delivery': lims_sample.udf['Data Analysis'],
            'sex': SEX_MAP.get(lims_sample.udf.get('Gender'), 'N/A'),
            'app_tag': lims_sample.udf['Sequencing Analysis'],
            'app_tag_version': int(lims_sample.udf.get('Application Tag Version', '1')),
            'priority': lims_sample.udf.get('priority', 'standard'),
            'capture_kit': capture_kit if capture_kit != 'NA' else None,
            'project': lims_sample.project.name,
            'source': lims_sample.udf.get('Source', 'N/A'),
        }
    except KeyError as error:
        log.error("missing UDF key for samples: %s", lims_sample.id)
        click.echo(lims_sample.udf.items(), err=True)
        raise error

    # parse artifacts
    for artifact in artifacts:
        if artifact.parent_process is None:
            continue
        elif artifact.parent_process.type.id == '33':
            process = artifact.parent_process
            data['capture_kit'] = process.udf['Capture Library version']
            data['library_prep_method'] = process.udf['Method document and version no:']
            data['library_prep_lotno'] = process.udf['Lot no: Capture library']
        elif artifact.parent_process.type.id == '667':
            # PCR Free library prep
            method_no = artifact.parent_process.udf['Method document']
            method_version = artifact.parent_process.udf['Method document version']
            data['library_prep_method'] = ":".join([method_no, method_version])
        elif artifact.parent_process.type.id == '663':
            # sequencing (cluster generation...)
            method_no = artifact.parent_process.udf['Method']
            method_version = artifact.parent_process.udf['Version']
            data['sequencing_method'] = ":".join([method_no, method_version])
            data['flowcell'] = artifact.container.name if artifact.container else None
        elif artifact.parent_process.type.id in ('670', '671'):
            # more seq (actual sequencing process)
            # or for EX: CG002 - Illumina Sequencing (Illumina SBS)
            if 'sequencing_date' not in data:
                # get the start date for sequenceing
                data['sequencing_date'] = parse_date(artifact.parent_process.date_run)
        elif artifact.parent_process.type.id == '159':
            # delivery
            delivery_date = artifact.parent_process.udf['Date delivered']
            data['delivery_date'] = datetime.combine(delivery_date, datetime.min.time())
            method_no = artifact.parent_process.udf['Method Document']
            method_version = artifact.parent_process.udf['Method Version']
            data['delivery_method'] = ':'.join([method_no, method_version])
        elif artifact.parent_process.type.id == '669':
            # CG002 - Hybridize Library  (SS XT)
            process = artifact.parent_process
            capture_kit_udf = 'SureSelect capture library/libraries used'
            data['capture_kit'] = process.udf[capture_kit_udf]
            method_no = process.udf['Method document']
            method_version = artifact.parent_process.udf['Method document versio']
            data['library_prep_method'] = ":".join([method_no, method_version])
        elif artifact.parent_process.type.id == '664':
            # CG002 - Cluster Generation (Illumina SBS)
            method_no = artifact.parent_process.udf['Method Document 1']
            method_version = artifact.parent_process.udf['Document 1 Version']
            data['sequencing_method'] = ":".join([method_no, method_version])
            raw_flowcell = artifact.parent_process.udf['Experiment Name']
            data['flowcell'] = raw_flowcell.split(' ')[0]

    return data


def consolidate_family(families):
    """Consolidate family data across multiple samples."""
    for index, family in enumerate(families):
        if index == 0:
            new_data = copy.deepcopy(family)
            gene_panels = set(new_data['gene_panels'])

        else:
            assert family['customer'] == new_data['customer']
            assert family['family_id'] == new_data['family_id']
            assert family['reference_genome'] == new_data['reference_genome']
            for gene_panel in family['gene_panels']:
                gene_panels.add(gene_panel)

    new_data['gene_panels'] = list(gene_panels)

    return new_data



# -*- coding: utf-8 -*-
import pytest
from cglims.exc import UnknownSequencingTypeError


def test_application(apptags):
    # GIVEN a application tag
    # WHEN parsing sequencing type
    # THEN is should reflect the type
    for apptag_type, apptag in apptags.iteritems():
        app_type = apptag.application
        if apptag_type == 'wgs':
            assert app_type == 'WGS'
        if apptag_type == 'wes':
            assert app_type == 'EXO'
        if apptag_type == 'microbial':
            assert app_type == 'MWG'
        if apptag_type == 'rna':
            assert app_type == 'RNL'
        if apptag_type == 'external':
            assert app_type == 'EXX'
        if apptag_type == 'external_wgs':
            assert app_type == 'WGX'
        if apptag_type == 'metagenome':
            assert app_type == 'MET'
        if apptag_type == 'rml':
            assert app_type == 'RML'
        if apptag_type == 'targeted':
            assert app_type == 'EFT'

def test_sequencing_type(apptags):
    for apptag_type, apptag in apptags.iteritems():
        if apptag_type == 'rml' or apptag_type == 'rna':
            with pytest.raises(UnknownSequencingTypeError):
                seq_type = apptag.sequencing_type
        else:
            seq_type = apptag.sequencing_type
        if apptag_type == 'wgs':
            assert seq_type == 'wgs'
        if apptag_type == 'wes':
            assert seq_type == 'wes'
        if apptag_type == 'external':
            assert seq_type == 'wes'
        if apptag_type == 'external_wgs':
            assert seq_type == 'wgs'
        if apptag_type == 'metagenome':
            assert seq_type == 'wgs'
        if apptag_type == 'microbial':
            assert seq_type == 'wgs'
        if apptag_type == 'targeted':
            assert seq_type == 'tga'

def test_sequencing_type_mip(apptags):
    for apptag_type, apptag in apptags.iteritems():
        if apptag_type == 'rml' or apptag_type == 'rna':
            with pytest.raises(UnknownSequencingTypeError):
                seq_type = apptag.sequencing_type_mip
        else:
            seq_type = apptag.sequencing_type_mip
        if apptag_type == 'wgs':
            assert seq_type == 'wgs'
        if apptag_type == 'wes':
            assert seq_type == 'wes'
        if apptag_type == 'external':
            assert seq_type == 'wes'
        if apptag_type == 'external_wgs':
            assert seq_type == 'wgs'
        if apptag_type == 'metagenome':
            assert seq_type == 'wgs'
        if apptag_type == 'microbial':
            assert seq_type == 'wgs'
        if apptag_type == 'targeted':
            assert seq_type == 'wes'

def test_library_prep(apptags):
    for apptag_type, apptag in apptags.iteritems():
        lib_prep = apptag.library_prep
        if apptag_type == 'wgs':
            assert lib_prep == 'PCF'
        if apptag_type == 'wes':
            assert lib_prep == 'SXT'
        if apptag_type == 'microbial':
            assert lib_prep == 'NXT'
        if apptag_type == 'rna':
            assert lib_prep == 'POA'
        if apptag_type == 'external':
            assert lib_prep == 'CUS'
        if apptag_type == 'external_wgs':
            assert lib_prep == 'CUS'
        if apptag_type == 'metagenome':
            assert lib_prep == 'PCF'
        if apptag_type == 'rml':
            assert lib_prep == 'P10'
        if apptag_type == 'targeted':
            assert lib_prep == 'SXT'

def test_is_human(apptags):
    for apptag_type, apptag in apptags.iteritems():
        is_human = apptag.is_human
        if apptag_type == 'wgs':
            assert is_human == True
        if apptag_type == 'wes':
            assert is_human == True
        if apptag_type == 'microbial':
            assert is_human == False
        if apptag_type == 'rna':
            assert is_human == False
        if apptag_type == 'external':
            assert is_human == True
        if apptag_type == 'external_wgs':
            assert is_human == True
        if apptag_type == 'metagenome':
            assert is_human == True
        if apptag_type == 'rml':
            assert is_human == False
        if apptag_type == 'targeted':
            assert is_human == True

def test_is_panel(apptags):
    for apptag_type, apptag in apptags.iteritems():
        is_panel = apptag.is_panel
        if apptag_type == 'wgs':
            assert is_panel == False
        if apptag_type == 'wes':
            assert is_panel == True
        if apptag_type == 'microbial':
            assert is_panel == False
        if apptag_type == 'rna':
            assert is_panel == False
        if apptag_type == 'external':
            assert is_panel == True
        if apptag_type == 'external_wgs':
            assert is_panel == False
        if apptag_type == 'metagenome':
            assert is_panel == False
        if apptag_type == 'rml':
            assert is_panel == False
        if apptag_type == 'targeted':
            assert is_panel == True

def test_is_microbial(apptags):
    for apptag_type, apptag in apptags.iteritems():
        is_microbial = apptag.is_microbial
        if apptag_type == 'wgs':
            assert is_microbial == False
        if apptag_type == 'wes':
            assert is_microbial == False
        if apptag_type == 'microbial':
            assert is_microbial == True
        if apptag_type == 'rna':
            assert is_microbial == False
        if apptag_type == 'external':
            assert is_microbial == False
        if apptag_type == 'external_wgs':
            assert is_microbial == False
        if apptag_type == 'metagenome':
            assert is_microbial == False
        if apptag_type == 'rml':
            assert is_microbial == False
        if apptag_type == 'targeted':
            assert is_microbial == False

def test_is_rna(apptags):
    for apptag_type, apptag in apptags.iteritems():
        is_rna = apptag.is_rna
        if apptag_type == 'wgs':
            assert is_rna == False
        if apptag_type == 'wes':
            assert is_rna == False
        if apptag_type == 'microbial':
            assert is_rna == False
        if apptag_type == 'rna':
            assert is_rna == True
        if apptag_type == 'external':
            assert is_rna == False
        if apptag_type == 'external_wgs':
            assert is_rna == False
        if apptag_type == 'metagenome':
            assert is_rna == False
        if apptag_type == 'rml':
            assert is_rna == False
        if apptag_type == 'targeted':
            assert is_rna == False

def test_is_external(apptags):
    for apptag_type, apptag in apptags.iteritems():
        is_external = apptag.is_external
        if apptag_type == 'wgs':
            assert is_external == False
        if apptag_type == 'wes':
            assert is_external == False
        if apptag_type == 'microbial':
            assert is_external == False
        if apptag_type == 'rna':
            assert is_external == False
        if apptag_type == 'external':
            assert is_external == True
        if apptag_type == 'external_wgs':
            assert is_external == True
        if apptag_type == 'metagenome':
            assert is_external == False
        if apptag_type == 'rml':
            assert is_external == False
        if apptag_type == 'targeted':
            assert is_external == False

def test_reads(apptags):
    for apptag_type, apptag in apptags.iteritems():
        reads = apptag.reads
        if apptag_type == 'wgs':
            assert reads == 10000000 * 30
        if apptag_type == 'wes':
            assert reads == 100000000
        if apptag_type == 'microbial':
            assert reads == 3000000
        if apptag_type == 'rna':
            assert reads == 30000000
        if apptag_type == 'external':
            assert reads == 0
        if apptag_type == 'external_wgs':
            assert reads == 0
        if apptag_type == 'metagenome':
            assert reads == 20000000
        if apptag_type == 'rml':
            assert reads == 150000000
        if apptag_type == 'targeted':
            assert reads == 20000000
